/*
 * aTunes 1.11.2
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.controls.playList;

import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;

import net.sourceforge.atunes.gui.Fonts;
import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.model.PlayListTableModel;
import net.sourceforge.atunes.gui.model.TransferableList;
import net.sourceforge.atunes.kernel.actions.Actions;
import net.sourceforge.atunes.kernel.actions.Add100RandomSongsAction;
import net.sourceforge.atunes.kernel.actions.Add100SongsMostPlayedAction;
import net.sourceforge.atunes.kernel.actions.Add100UnplayedSongsAction;
import net.sourceforge.atunes.kernel.actions.Add10AlbumsMostPlayedAction;
import net.sourceforge.atunes.kernel.actions.Add10ArtistsMostPlayedAction;
import net.sourceforge.atunes.kernel.actions.Add10RandomSongsAction;
import net.sourceforge.atunes.kernel.actions.Add10SongsMostPlayedAction;
import net.sourceforge.atunes.kernel.actions.Add10UnplayedSongsAction;
import net.sourceforge.atunes.kernel.actions.Add1AlbumsMostPlayedAction;
import net.sourceforge.atunes.kernel.actions.Add1ArtistsMostPlayedAction;
import net.sourceforge.atunes.kernel.actions.Add50RandomSongsAction;
import net.sourceforge.atunes.kernel.actions.Add50SongsMostPlayedAction;
import net.sourceforge.atunes.kernel.actions.Add50UnplayedSongsAction;
import net.sourceforge.atunes.kernel.actions.Add5AlbumsMostPlayedAction;
import net.sourceforge.atunes.kernel.actions.Add5ArtistsMostPlayedAction;
import net.sourceforge.atunes.kernel.actions.AutoSetCoversAction;
import net.sourceforge.atunes.kernel.actions.AutoSetGenresAction;
import net.sourceforge.atunes.kernel.actions.AutoSetLyricsAction;
import net.sourceforge.atunes.kernel.actions.AutoSetTitlesAction;
import net.sourceforge.atunes.kernel.actions.AutoSetTracksAction;
import net.sourceforge.atunes.kernel.actions.ClearPlayListAction;
import net.sourceforge.atunes.kernel.actions.CreatePlayListWithSelectedAlbumsAction;
import net.sourceforge.atunes.kernel.actions.CreatePlayListWithSelectedArtistsAction;
import net.sourceforge.atunes.kernel.actions.EditTagAction;
import net.sourceforge.atunes.kernel.actions.LoadPlayListAction;
import net.sourceforge.atunes.kernel.actions.MoveDownAction;
import net.sourceforge.atunes.kernel.actions.MoveToBottomAction;
import net.sourceforge.atunes.kernel.actions.MoveToTopAction;
import net.sourceforge.atunes.kernel.actions.MoveUpAction;
import net.sourceforge.atunes.kernel.actions.OpenFolderAction;
import net.sourceforge.atunes.kernel.actions.PlayAction;
import net.sourceforge.atunes.kernel.actions.RemoveFromPlayListAction;
import net.sourceforge.atunes.kernel.actions.SavePlayListAction;
import net.sourceforge.atunes.kernel.actions.SetPlayListSelectionAsFavoriteAlbumAction;
import net.sourceforge.atunes.kernel.actions.SetPlayListSelectionAsFavoriteArtistAction;
import net.sourceforge.atunes.kernel.actions.SetPlayListSelectionAsFavoriteSongAction;
import net.sourceforge.atunes.kernel.actions.ShowPlayListItemInfoAction;
import net.sourceforge.atunes.kernel.actions.ShufflePlayListAction;
import net.sourceforge.atunes.kernel.actions.TogglePlayListFilterAction;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class PlayListTable.
 */
public class PlayListTable extends JTable implements DragSourceListener, DragGestureListener {

	/**
	 * The Enum PlayState.
	 */
	public enum PlayState {

		/** The STOPPED. */
		STOPPED,

		/** The PLAYING. */
		PLAYING,

		/** The PAUSED. */
		PAUSED
	}

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 9209069236823917569L;

	/** The playing song. */
	private int playingSong;

	/** The play state. */
	private PlayState playState = PlayState.STOPPED;

	/** The menu. */
	private JPopupMenu menu;

	/** The right menu. */
	JPopupMenu rightMenu;

	/** The arrange columns. */
	private JMenuItem arrangeColumns;

	/** The listeners. */
	List<PlayListColumnClickedListener> listeners = new ArrayList<PlayListColumnClickedListener>();

	/**
	 * Drag source for this play list to drag songs to device
	 */
	private DragSource dragSource;

	/**
	 * Instantiates a new play list table.
	 */
	public PlayListTable() {
		super();
		setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		setShowGrid(false);

		// Add mouse click listener on table header
		getTableHeader().addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				// Use left button to sort
				if (e.getButton() == MouseEvent.BUTTON1) {
					PlayListColumnModel colModel = (PlayListColumnModel) getColumnModel();
					int column = colModel.getColumnIndexAtPosition(e.getX());
					if (column >= 0) {
						Column columnClicked = ((PlayListTableModel) getModel()).getColumn(column);
						for (int i = 0; i < listeners.size(); i++) {
							listeners.get(i).columnClicked(columnClicked);
						}
					}
				}
				// Use right button to arrange columns
				else if (e.getButton() == MouseEvent.BUTTON3) {
					rightMenu.show(PlayListTable.this, e.getX(), e.getY());
				}
			}
		});

		// Set table model
		setModel(new PlayListTableModel(this));

		// Set column model
		setColumnModel(new PlayListColumnModel(this));

		// Disable autoresize, as we will control it
		setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

		// Set renderers
		PlayListRenderers.addRenderers(this);

		// Remove enter key event, which moves selection down
		getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "none");

		rightMenu = new JPopupMenu();
		arrangeColumns = setMenuItem(new JMenuItem(LanguageTool.getString("ARRANGE_COLUMNS")));
		rightMenu.add(arrangeColumns);

		menu = new JPopupMenu();

		menu.add(setMenuItem(new JMenuItem(Actions.getAction(PlayAction.class))));
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(ShowPlayListItemInfoAction.class))));
		menu.add(new JSeparator());
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(OpenFolderAction.class))));
		menu.add(new JSeparator());
		JMenu tagMenu = setMenu(new JMenu(LanguageTool.getString("TAGS")));
		tagMenu.add(setMenuItem(new JMenuItem(Actions.getAction(EditTagAction.class))));
		tagMenu.add(setMenuItem(new JMenuItem(Actions.getAction(AutoSetLyricsAction.class))));
		tagMenu.add(setMenuItem(new JMenuItem(Actions.getAction(AutoSetTitlesAction.class))));
		tagMenu.add(setMenuItem(new JMenuItem(Actions.getAction(AutoSetTracksAction.class))));
		tagMenu.add(setMenuItem(new JMenuItem(Actions.getAction(AutoSetGenresAction.class))));
		tagMenu.add(setMenuItem(new JMenuItem(Actions.getAction(AutoSetCoversAction.class))));
		menu.add(tagMenu);
		menu.add(new JSeparator());
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(SavePlayListAction.class))));
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(LoadPlayListAction.class))));
		menu.add(new JSeparator());
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(RemoveFromPlayListAction.class))));
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(ClearPlayListAction.class))));
		menu.add(new JSeparator());
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(MoveToTopAction.class))));
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(MoveUpAction.class))));
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(MoveDownAction.class))));
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(MoveToBottomAction.class))));
		menu.add(new JSeparator());
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(ShufflePlayListAction.class))));
		menu.add(new JSeparator());
		JMenu smartPlayList = new JMenu(LanguageTool.getString("SMART_PLAYLIST"));
		smartPlayList.add(new JMenuItem(Actions.getAction(Add10RandomSongsAction.class)));
		smartPlayList.add(new JMenuItem(Actions.getAction(Add50RandomSongsAction.class)));
		smartPlayList.add(new JMenuItem(Actions.getAction(Add100RandomSongsAction.class)));
		smartPlayList.add(new JSeparator());
		smartPlayList.add(new JMenuItem(Actions.getAction(Add10SongsMostPlayedAction.class)));
		smartPlayList.add(new JMenuItem(Actions.getAction(Add50SongsMostPlayedAction.class)));
		smartPlayList.add(new JMenuItem(Actions.getAction(Add100SongsMostPlayedAction.class)));
		smartPlayList.add(new JSeparator());
		smartPlayList.add(new JMenuItem(Actions.getAction(Add1AlbumsMostPlayedAction.class)));
		smartPlayList.add(new JMenuItem(Actions.getAction(Add5AlbumsMostPlayedAction.class)));
		smartPlayList.add(new JMenuItem(Actions.getAction(Add10AlbumsMostPlayedAction.class)));
		smartPlayList.add(new JSeparator());
		smartPlayList.add(new JMenuItem(Actions.getAction(Add1ArtistsMostPlayedAction.class)));
		smartPlayList.add(new JMenuItem(Actions.getAction(Add5ArtistsMostPlayedAction.class)));
		smartPlayList.add(new JMenuItem(Actions.getAction(Add10ArtistsMostPlayedAction.class)));
		smartPlayList.add(new JSeparator());
		smartPlayList.add(new JMenuItem(Actions.getAction(Add10UnplayedSongsAction.class)));
		smartPlayList.add(new JMenuItem(Actions.getAction(Add50UnplayedSongsAction.class)));
		smartPlayList.add(new JMenuItem(Actions.getAction(Add100UnplayedSongsAction.class)));
		menu.add(smartPlayList);
		menu.add(new JSeparator());
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(SetPlayListSelectionAsFavoriteSongAction.class))));
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(SetPlayListSelectionAsFavoriteAlbumAction.class))));
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(SetPlayListSelectionAsFavoriteArtistAction.class))));
		menu.add(new JSeparator());
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(CreatePlayListWithSelectedArtistsAction.class))));
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(CreatePlayListWithSelectedAlbumsAction.class))));
		menu.add(new JSeparator());
		menu.add(setMenuItem(new JMenuItem(Actions.getAction(TogglePlayListFilterAction.class))));
		GuiUtils.applyComponentOrientation(menu);

		// Create drag source and set listener
		dragSource = new DragSource();
		dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_COPY, this);

	}

	/**
	 * Adds the play list column clicked listener.
	 * 
	 * @param l
	 *            the l
	 */
	public void addPlayListColumnClickedListener(PlayListColumnClickedListener l) {
		listeners.add(l);
	}

	/**
	 * Gets the arrange columns.
	 * 
	 * @return the arrangeColumns
	 */
	public JMenuItem getArrangeColumns() {
		return arrangeColumns;
	}

	/**
	 * Gets the menu.
	 * 
	 * @return the menu
	 */
	public JPopupMenu getMenu() {
		return menu;
	}

	/**
	 * Gets the playing song.
	 * 
	 * @return the playing song
	 */
	public int getPlayingSong() {
		return playingSong;
	}

	/**
	 * Gets the play state.
	 * 
	 * @return the play state
	 */
	public PlayState getPlayState() {
		return playState;
	}

	/**
	 * Sets the menu.
	 * 
	 * @param item
	 *            the item
	 * 
	 * @return the j menu
	 */
	private JMenu setMenu(JMenu item) {
		item.setFont(Fonts.PLAY_LIST_FONT);
		return item;
	}

	/**
	 * Sets the menu item.
	 * 
	 * @param item
	 *            the item
	 * 
	 * @return the j menu item
	 */
	private JMenuItem setMenuItem(JMenuItem item) {
		item.setFont(Fonts.PLAY_LIST_FONT);
		return item;
	}

	/**
	 * Sets the playing song.
	 * 
	 * @param row
	 *            the new playing song
	 */
	public void setPlayingSong(int row) {
		playingSong = row;
		((PlayListTableModel) getModel()).refreshTable();
	}

	/**
	 * Sets the play state.
	 * 
	 * @param playState
	 *            the new play state
	 */
	public void setPlayState(PlayState playState) {
		this.playState = playState;
		((PlayListTableModel) getModel()).refreshTable();
	}

	@Override
	public void dragDropEnd(DragSourceDropEvent dsde) {
		// TODO Auto-generated method stub

	}

	@Override
	public void dragEnter(DragSourceDragEvent dsde) {
		// TODO Auto-generated method stub

	}

	@Override
	public void dragExit(DragSourceEvent dse) {
		// TODO Auto-generated method stub

	}

	@Override
	public void dragGestureRecognized(DragGestureEvent dge) {
		// Get selected rows, add AudioFile objects to a list and start a drag event
		int[] selectedRows = getSelectedRows();
		List<Object> itemsToDrag = new ArrayList<Object>();
		for (int i : selectedRows) {
			AudioObject obj = ((PlayListTableModel) getModel()).getAudioObjectAt(i);
			if (obj instanceof AudioFile) {
				itemsToDrag.add(obj);
			}
		}
		TransferableList<Object> items = new TransferableList<Object>(itemsToDrag);
		dragSource.startDrag(dge, DragSource.DefaultCopyDrop, items, this);
	}

	@Override
	public void dragOver(DragSourceDragEvent dsde) {
		// TODO Auto-generated method stub

	}

	@Override
	public void dropActionChanged(DragSourceDragEvent dsde) {
		// TODO Auto-generated method stub

	}
}
