/*
 * aTunes 1.11.2
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.actions;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.AbstractAction;

import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.model.NavigationTableModel;
import net.sourceforge.atunes.gui.views.dialogs.CopyProgressDialog;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.executors.ImportProcess;
import net.sourceforge.atunes.kernel.executors.processes.ExportFilesProcess;
import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

public class CopyToRepositoryAction extends AbstractAction {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2416674807979541242L;

	CopyToRepositoryAction() {
		super(LanguageTool.getString("COPY_TO_REPOSITORY"), ImageLoader.EXPORT);
		putValue(SHORT_DESCRIPTION, LanguageTool.getString("COPY_TO_REPOSITORY"));
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		List<AudioObject> songs = ((NavigationTableModel) ControllerProxy.getInstance().getNavigationController().getNavigationPanel().getNavigationTable().getModel()).getSongs();
		int filesToExport = songs.size();
		CopyProgressDialog copyProgressDialog = VisualHandler.getInstance().getCopyProgressDialog();
		copyProgressDialog.getTotalFilesLabel().setText(StringUtils.getString(filesToExport));
		copyProgressDialog.getProgressBar().setMaximum(filesToExport);
		copyProgressDialog.setVisible(true);
		final ImportProcess importer = new ImportProcess(AudioFile.getAudioFiles(songs), ExportFilesProcess.FULL_STRUCTURE, null);
		copyProgressDialog.getCancelButton().addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				importer.notifyCancel();
			}
		});
		importer.start();
	}

}
