/*
 * aTunes 1.11.2
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.playerControls;

import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.views.panels.PlayerControlsPanel;
import net.sourceforge.atunes.kernel.controllers.model.PanelController;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.TimeUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;

/**
 * The Class PlayerControlsController.
 */
public class PlayerControlsController extends PanelController<PlayerControlsPanel> {

	/**
	 * Instantiates a new player controls controller.
	 * 
	 * @param panel
	 *            the panel
	 */
	public PlayerControlsController(PlayerControlsPanel panel) {
		super(panel);
		addBindings();
		addStateBindings();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
	 */
	@Override
	protected void addBindings() {
		PlayerControlsListener listener = new PlayerControlsListener(getPanelControlled(), this);

		getPanelControlled().getProgressBar().addMouseListener(listener);
		getPanelControlled().getPlayButton().addActionListener(listener);
		getPanelControlled().getStopButton().addActionListener(listener);
		getPanelControlled().getNextButton().addActionListener(listener);
		getPanelControlled().getPreviousButton().addActionListener(listener);
		getPanelControlled().getVolumeButton().addActionListener(listener);
		getPanelControlled().getVolumeSlider().addChangeListener(listener);
		getPanelControlled().getBalanceSlider().addChangeListener(listener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings
	 * ()
	 */
	@Override
	protected void addStateBindings() {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
	 */
	@Override
	protected void notifyReload() {
		// Nothing to do
	}

	/**
	 * Sets the max duration.
	 * 
	 * @param length
	 *            the new length
	 */
	public void setAudioObjectLength(long length) {
		getLogger().debug(LogCategories.CONTROLLER, new String[] { Long.toString(length) });

		getPanelControlled().getProgressBar().setMaximum((int) length);
	}

	/**
	 * Sets the mute.
	 * 
	 * @param mute
	 *            the new mute
	 */
	public void setMute(boolean mute) {
		getPanelControlled().getVolumeButton().setSelected(mute);
		if (mute) {
			getPanelControlled().getVolumeButton().setIcon(ImageLoader.VOLUME_MUTE);
		} else {
			int value = getPanelControlled().getVolumeSlider().getValue();
			if (value > 80) {
				getPanelControlled().getVolumeButton().setIcon(ImageLoader.VOlUME_MAX);
			} else if (value > 40) {
				getPanelControlled().getVolumeButton().setIcon(ImageLoader.VOLUME_MED);
			} else if (value > 5) {
				getPanelControlled().getVolumeButton().setIcon(ImageLoader.VOLUME_MIN);
			} else {
				getPanelControlled().getVolumeButton().setIcon(ImageLoader.VOLUME_ZERO);
			}
		}
	}

	/**
	 * Sets the playing.
	 * 
	 * @param playing
	 *            the new playing
	 */
	public void setPlaying(boolean playing) {
		getPanelControlled().setPlaying(playing);
	}

	/**
	 * Sets the slidable.
	 * 
	 * @param slidable
	 *            the new slidable
	 */
	public void setSlidable(boolean slidable) {
		getPanelControlled().getProgressBar().setEnabled(slidable);
	}

	/**
	 * Sets the time.
	 * 
	 * @param timePlayed
	 *            the time played
	 * @param totalTime
	 *            the total time
	 */
	public void setCurrentAudioObjectTimePlayed(long timePlayed, long totalTime) {
		long remainingTime = totalTime - timePlayed;
		if (timePlayed == 0) {
			getPanelControlled().getRemainingTime().setText(TimeUtils.milliseconds2String(timePlayed));
		} else {
			getPanelControlled().getRemainingTime().setText(remainingTime > 0 ? StringUtils.getString("- ", TimeUtils.milliseconds2String(remainingTime)) : "-");
		}

		getPanelControlled().getTime().setText(TimeUtils.milliseconds2String(timePlayed));
		getPanelControlled().getProgressBar().setValue((int) timePlayed);
	}

	/**
	 * Sets the volume.
	 * 
	 * @param value
	 *            the new volume
	 */
	public void setVolume(int value) {
		getPanelControlled().getVolumeSlider().setValue(value);
		getPanelControlled().getVolumeLevel().setText(StringUtils.getString(String.valueOf(value), " %"));
	}

}
