/*
 * aTunes 1.11.2
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.executors.processes;

import java.util.List;

import javax.swing.SwingUtilities;

import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.tags.tag.EditTagInfo;
import net.sourceforge.atunes.kernel.modules.repository.tags.tag.Tag;
import net.sourceforge.atunes.kernel.modules.repository.tags.writer.TagModifier;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class ChangeTagsProcess.
 */
public class ChangeTagsProcess implements Runnable {

	/** The logger. */
	private Logger logger = new Logger();

	/** The files to edit. */
	List<AudioFile> audioFilesToEdit;

	/** The edit tag info. */
	private EditTagInfo editTagInfo;

	/**
	 * Process for writting tag to files. Receives AudioFiles to be written and
	 * new properties (metainformation)
	 * 
	 * @param audioFilesToEdit
	 *            Files that must be edited (tag)
	 * @param editTagInfo
	 *            Information to be written
	 */
	public ChangeTagsProcess(List<AudioFile> audioFilesToEdit, EditTagInfo editTagInfo) {
		super();
		this.audioFilesToEdit = audioFilesToEdit;
		this.editTagInfo = editTagInfo;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run() {
		try {
			logger.debug(LogCategories.PROCESS);

			VisualHandler.getInstance().showIndeterminateProgressDialog(StringUtils.getString(LanguageTool.getString("PERFORMING_CHANGES"), "..."));

			for (int i = 0; i < audioFilesToEdit.size(); i++) {
				Tag newTag = AudioFile.getNewTag(audioFilesToEdit.get(i), editTagInfo);
				Tag oldTag = (audioFilesToEdit.get(i)).getTag();

				byte[] c = null;
				boolean shouldEditCover = editTagInfo.isTagEdited("COVER");
				if (shouldEditCover) {
					Object cover = editTagInfo.get("COVER");
					if (cover != null) {
						c = (byte[]) cover;
					}
					newTag.setInternalImage((oldTag != null && oldTag.hasInternalImage() && !shouldEditCover) || (shouldEditCover && cover != null));
				}

				TagModifier.setInfo(audioFilesToEdit.get(i), newTag, shouldEditCover, c);
			}

			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					TagModifier.refreshAfterTagModify(audioFilesToEdit);
				}
			});
		} finally {
			VisualHandler.getInstance().hideIndeterminateProgressDialog();
		}
	}
}
